<?php

namespace Fastrack\Core\Api;

use Fastrack\Core\Model\Imgix;
use Fastrack\Core\Util\Cache;
use Psr\Container\NotFoundExceptionInterface;
use Psr\SimpleCache\InvalidArgumentException;
use SilverStripe\Assets\Image;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Control\HTTPResponse;
use SilverStripe\Control\Middleware\HTTPCacheControlMiddleware;
use SilverStripe\Core\Config\Config;
use SilverStripe\Security\Security;

class Api
{

	private static array $image_sizes = [
		'sm' => 300,
		'md' => 992,
		'lg' => 1920,
	];

	private static int $icon_width = 300;

	public static function constructResponse(Controller $controller, array $data): HTTPResponse
	{
		HTTPCacheControlMiddleware::singleton()->enableCache(true, 60 * 60);
		$controller->getResponse()->setBody(json_encode($data));
		$controller->getResponse()->addHeader("Content-type", "application/json");
		return $controller->getResponse();
	}


	public static function fetchFromCache($key, $default)
	{
		if (!self::shouldCache()) {
			return $default;
		}
		try {
			return Cache::getCache()->get($key, $default);
		} catch (NotFoundExceptionInterface|InvalidArgumentException $e) {
			return $default;
		}
	}

	public static function setCache(string $key, $value): void
	{
		if (!self::shouldCache()) {
			return;
		}
		try {
			Cache::getCache()->set($key, $value, 60 * 60 * 6);
		} catch (NotFoundExceptionInterface|InvalidArgumentException $e) {
			user_error($e->getMessage(), E_USER_WARNING);
		}
	}

	public static function shouldCache(): bool
	{
		if (Security::getCurrentUser()) {
			return false;
		}
		if (Director::isDev()) {
			return false;
		}
		return true;
	}

	public static function buildImage(Imgix|Image|null $image): ?array
	{
		if (!$image || !$image->ID) {
			return null;
		}
		$sizes = Config::inst()->get(self::class, 'image_sizes');
		$data = ['alt' => $image->Title];
		foreach	($sizes as $key => $size) {
			$data[$key] = $image->ScaleMaxWidth($size)->Link();
		}
		return $data;
	}

	public static function buildGallery($model, string $galleryKey, int $maxImages = 4): array
	{
		$slides = [];
		$gallery = $model->Galleries()->find('Key', $galleryKey);
		if ($gallery) {
			foreach ($gallery->Slides() as $slide) {
				if (count($slides) === $maxImages) {
					break;
				}
				$image = Api::buildImage($slide->Image());
				if ($image) {
					$slides[] = [
						'title' => $slide->Title,
						'image' => Api::buildImage($slide->Image()),
					];
				}
			}
		}
		return $slides;
	}

}
