<?php

namespace Fastrack\Core\Api;


use Fastrack\Core\Model\FeatureGroup;
use Fastrack\Core\Model\Offer;
use Fastrack\Core\Model\Property;
use Fastrack\Core\Model\Room;
use SilverStripe\CMS\Controllers\ContentController;
use SilverStripe\Control\HTTPResponse;
use SilverStripe\Control\HTTPResponse_Exception;
use SilverStripe\ORM\DataObject;

class PropertyApi extends ContentController
{

	public const PROPERTIES_LISTING = '/api/properties';

	private const QUERY_PARAM_PROPERTY_ID = 'property-id';

	/**
	 * @config
	 */
	private static string $room_gallery_key = 'gallery';

	/**
	 * @config
	 */
	private static string $property_gallery_key = 'gallery';

	/**
	 * Array of statistics of a property with 'name', 'key' and 'icon'
	 * For example:
	 *  [
	 * 		['name' => 'Villas', 'key' => 'villas', 'icon' => '/images/wbi/property-fill-icon.svg'],
	 * ]
	 *
	 * @config
	 */
	private static array $property_key_feature_settings = [];

	/**
	 * Key room features
	 * Refer to $property_key_feature_settings for specification
	 *
	 * @config
	 */
	private static array $room_key_feature_settings = [];

	/**
	 * Property for feature groups
	 *
	 * name: json key
	 * feature group key: feature group to search for
	 *
	 * for example:
	 * [
	 *        ['name' => 'usps', 'featureGroupKey' => 'unique-selling-points'],
	 * ]
	 *
	 * @config
	 */
	private static array $property_feature_groups = [];

	/**
	 * Room for feature groups.
	 * Refer to $property_feature_groups for array structure
	 *
	 * @config
	 */
	private static array $room_feature_groups = [];

	/**
	 * @config
	 */
	private static int $max_gallery_images = 4;

	private static array $allowed_actions = [
		'integration',
		'offers',
		'places',
		'properties',
		'property',
	];

	private static array $url_handlers = [
		'' => 'properties',
		'integration/$PropertyID!' => 'integration',
		'offers' => 'offers',
		'$PropertySlug!/places' => 'places',
		'$PropertySlug!' => 'property',
	];

	public function properties(): HTTPResponse
	{
		$CACHE_KEY = 'properties';
		$properties = Api::fetchFromCache($CACHE_KEY, []);
		if (!count($properties)) {
			foreach (Property::get() as $property) {
				$properties[] = $this->buildProperty($property);
			}
			Api::setCache($CACHE_KEY, $properties);
		}
		return Api::constructResponse($this, $properties);
	}




	/**
	 * @throws HTTPResponse_Exception
	 */
	public function property(): HTTPResponse
	{
		$property = $this->getProperty();
		return Api::constructResponse($this, $this->buildProperty($property));
	}

	/**
	 * @return HTTPResponse
	 */
	public function offers(): HTTPResponse
	{
		$CACHE_KEY = 'offers';
		$data = Api::fetchFromCache($CACHE_KEY, []);
		if (!count($data)) {
			foreach	(Offer::getValidOffers() as $offer) {
				$data[] = $this->buildOffer($offer);
			}
			Api::setCache($CACHE_KEY, $data);
		}
		return Api::constructResponse($this, $data);
	}

	/**
	 * @throws HTTPResponse_Exception
	 */
	public function places(): HTTPResponse
	{
		$property = $this->getProperty();
		$CACHE_KEY = 'places.' . $property->Slug;
		$places = Api::fetchFromCache($CACHE_KEY, []);
		if (!count($places)) {
			foreach ($property->Places() as $place) {
				$placeData = PlacesApi::buildPlace($place);
				$this->extend('updateBuildPlace', $placeData, $place);
				$places[] = $placeData;
			}
			Api::setCache($CACHE_KEY, $places);
		}
		return Api::constructResponse($this, $places);
	}

	/**
	 * @throws HTTPResponse_Exception
	 */
	public function integration(): HTTPResponse
	{
		$propertyId = intval($this->getRequest()->param('PropertyID'));
		if ($propertyId === 0) $this->httpError(400, 'Invalid property ID');
		$CACHE_KEY = 'property.integration.'.$propertyId;
		$cached = Api::fetchFromCache($CACHE_KEY, null);
		if (is_null($cached)) {
			/** @var Property $property */
			$property = Property::get()->find('PmsPropertyId', $propertyId);
			if (is_null($property)) $this->httpError(404);
			$cached = $this->buildProperty($property);
			Api::setCache($CACHE_KEY, $cached);
		}
		return Api::constructResponse($this, $cached);
	}


	/**
	 * @throws HTTPResponse_Exception
	 */
	private function getProperty(): Property
	{
		$propertySlug = $this->getRequest()->param('PropertySlug');
		$CACHE_KEY = 'property.' . $propertySlug;
		$property = Api::fetchFromCache($CACHE_KEY, null);
		if (is_null($property)) {
			/** @var Property $property */
			$property = Property::get()->find('Slug', $propertySlug);
			if (is_null($property)) {
				$this->httpError(404);
			}
			Api::setCache($CACHE_KEY, $property);
		}
		return $property;
	}

	private function buildProperty(Property $property): array
	{
		$rooms = [];
		foreach ($property->Rooms() as $room) {
			$rooms[] = $this->buildRoom($room);
		}
		$propertyData = [
			'id' => $property->ID,
			'bookingEngineId' => $property->BookingEngineId,
			'bookingLink' => $property->ConstructBookingLink(),
			'name' => $property->Title,
			'slug' => $property->Slug,
			'rooms' => $rooms,
			'fromPrice' => 0,
			'phone' => $property->getField('Phone'),
			'email' => $property->getField('Email'),
			'excerpt' => $property->Excerpt,
			'content' => $property->Content,
			'image' => Api::buildImage($property->Image()),
			'slides' => Api::buildGallery($property, $this->config()->get('property_gallery_key'), (int) $this->config()->get('max_galleryImages')),
			'address' => $this->buildAddress($property),
		];
		$this->addFeatureGroups($propertyData, $property);
		$this->extend('updateBuildProperty', $propertyData, $property);
		return $propertyData;
	}

	private function buildRoom(Room $room): array
	{
		$bookingEngineIds = [];
		if ($room->BookingEngineId) {
			foreach (explode(',', trim($room->BookingEngineId)) as $id) {
				$bookingEngineIds[] = intval(trim($id));
			}
		}
		$roomData = [
			'id' => $room->ID,
			'bookingLink' => $room->ConstructBookingLink(),
			'bookingEngineIds' => $bookingEngineIds,
			'name' => $room->Title,
			'slug' => $room->Slug,
			'bathrooms' => $room->NumberOfBathrooms,
			'bedrooms' => $room->NumberOfRooms,
			'guests' => $room->NumberOfGuests,
			'order' => $room->Sort,
			'excerpt' => $room->Excerpt,
			'content' => $room->Content,
			'image' => Api::buildImage($room->Image()),
			'slides' => Api::buildGallery($room, $this->config()->get('room_gallery_key'), (int) $this->config()->get('max_gallery_images')),
			'keyFeatures' => $this->getKeyFeatures($room, $this->config()->get('room_key_feature_settings'))
		];
		$this->addFeatureGroups($roomData, $room);
		$this->extend('updateBuildRoom', $roomData, $room);
		return $roomData;
	}




	private function buildAddress(Property $property): array
	{
		return [
			'address' => $property->getAddressFormatted(),
			'latitude' => floatval($property->Latitude),
			'longitude' => floatval($property->Longitude),
		];
	}

	private function getKeyFeatures(DataObject $model, array $settings): array
	{
		$features = [];
		foreach ($settings as $setting) {
			$features[] = [
				'name' => $setting['name'],
				'value' => $model->getField($setting['key']),
				'icon' => $setting['icon'],
			];
		}
		return $features;
	}

	private function addFeatureGroups(array &$data, Property|Room $model): void
	{
		$keys = (get_class($model) === Property::class) ?
			$this->config()->get('property_feature_groups') :
			$this->config()->get('room_feature_groups');

		foreach ($keys as $key) {
			$data[$key['name']] = $this->buildFeatureGroup($model, $key['featureGroupKey']);
		}
	}

	private function buildFeatureGroup(Property|Room $model, $key): array
	{
		$features = [];
		/** @var FeatureGroup $featureGroup */
		$featureGroup = $model->FeatureGroups()->find('Key', $key);
		if ($featureGroup) {
			foreach ($featureGroup->Features() as $featureItem) {
				$features[] = [
					'name' => $featureItem->getField('Name'),
					'image' => Api::buildImage($featureItem->Image()),
				];
			}
		}
		return $features;
	}

	private function buildOffer(Offer $offer): array
	{
		$properties = [];
		/** @var Property $property */
		foreach ($offer->Properties() as $property) {
			$properties[] = [
				'id' => $property->ID,
				'name' => $property->Title,
			];
		}
		$data = [
			'id' => $offer->ID,
			'slug' => $offer->Slug,
			'name' => $offer->Title,
			'bookingLink' => $offer->Url,
			'start' => $offer->Start,
			'end' => $offer->End,
			'excerpt' => $offer->Excerpt,
			'content' => $offer->Content,
			'image' => Api::buildImage($offer->Image()),
			'terms' => $offer->Terms,
			'properties' => $properties,
		];
		$this->extend('updateBuildOffer', $data, $offer);
		return $data;
	}

}
