<?php

namespace Fastrack\Core\Model;

use DateTime;
use Fastrack\Core\Extension\ContactExtension;
use Fastrack\Core\Extension\ContentExtension;
use Fastrack\Core\Extension\LocationExtension;
use Fastrack\Core\Extension\SecondaryTabSet;
use Fastrack\Core\Util\ModelUtilities;
use SilverStripe\LinkField\Models\Link;
use SilverStripe\LinkField\Form\LinkField;
use SilverStripe\Forms\CheckboxField;
use SilverStripe\Forms\CurrencyField;
use SilverStripe\Forms\DateField;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\TextField;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\FieldType\DBCurrency;
use SilverStripe\ORM\FieldType\DBDate;
use SilverStripe\ORM\ManyManyList;
use SilverStripe\ORM\ValidationResult;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\TagField\TagField;
use SilverStripe\Versioned\Versioned;
use SilverStripe\Forms\RequiredFields;
use TractorCow\Fluent\Extension\FluentVersionedExtension;

/**
 * @property string $Title
 * @property bool $Featured
 * @property DBCurrency $Price
 * @property string $Venue
 * @property string $StartAt
 * @property string $FinishAt
 * @method Gallery Galleries()
 * @method Link CallToAction()
 * @method ManyManyList Categories()
 */
class CalendarEvent extends DataObject implements PermissionProvider
{

	const PERMISSIONS_KEY = 'MANAGE_CALENDAR_EVENTS';

	private static string $table_name = 'CalendarEvent';

	private static array $extensions = [
		ContentExtension::class,
		ContactExtension::class,
		LocationExtension::class,
		Versioned::class,
		FluentVersionedExtension::class,
	];

	private static array $db = [
		'Title' => 'Varchar',
		'Featured' => 'Boolean',
		'Price' => 'Currency',
		'Venue' => 'Varchar',
		'StartAt' => 'Date',
		'FinishAt' => 'Date',
		'Time' => 'Varchar',
		'Sort' => 'Int',
	];

	private static array $has_one = [
		'CallToAction' => Link::class,
	];

	private static array $has_many = [
		'Galleries' => Gallery::class,
	];

	private static array $many_many = [
		'Categories' => CalendarEventCategory::class,
	];

	private static array $owns = [
		'Categories',
	];

	private static array $translate = [
		'Title',
	];

	private static array $hidden_fields = [
		'CallToActionID',
		'Sort',
	];

	private static string $default_sort = 'Sort';

	public function getCMSFields(): FieldList
	{
		$this->beforeUpdateCMSFields(function (FieldList $fields) {
			SecondaryTabSet::addFields($fields, [
				TextField::create('Title'),
				CurrencyField::create('Price'),
				TextField::create('Venue'),
				DateField::create('StartAt')->setDescription('*Required'),
				DateField::create('FinishAt')->setDescription('*Required'),
				TextField::create('Time'),
				LinkField::create('CallToAction'),
				TagField::create('Categories', 'Categories', CalendarEventCategory::get(), $this->Categories())
					->setCanCreate(true)
					->setShouldLazyLoad(true),
				CheckboxField::create('Featured'),
			]);
		});

		$fields = parent::getCMSFields();
		ModelUtilities::hideFields($this, $fields);
		return $fields;
	}

	public function getCMSValidator(): RequiredFields
	{
		return new RequiredFields(['Title']);
	}

	protected function onAfterWrite(): void
	{
		Gallery::createDefaultGalleries($this);
		parent::onAfterWrite();
	}

	public function providePermissions(): array
	{
		$permissions = [
			self::PERMISSIONS_KEY => [
				'name' => 'Manage calendar events',
				'category' => 'Calendar events',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canEdit($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canDelete($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canCreate($member = null, $context = array()): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function validate(): ValidationResult
	{
		$result = parent::validate();
		if (!$this->StartAt) {
			$result->addError('Start field is required');
		}
		if (!$this->FinishAt) {
			$result->addError('Finish field is required');
		}
		return $result;
	}

	public function getIsSameDayEvent(): bool
	{
		if (!$this->StartAt || !$this->FinishAt) {
			return false;
		}
		return $this->StartAt === $this->FinishAt;
	}

	public static function getValidationFilter(): array
	{
		return [
			'FinishAt:GreaterThanOrEqual' => (new DateTime('now'))->format('Y-m-d'),
		];
	}


}
