<?php

namespace Fastrack\Core\Model;

use Fastrack\Core\Extension\ContentExtension;
use Fastrack\Core\Extension\SecondaryTabSet;
use Fastrack\Core\Util\ModelUtilities;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Forms\CheckboxField;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\TextField;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\HasManyList;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Versioned\Versioned;
use TractorCow\Fluent\Extension\FluentVersionedExtension;

/**
 * @property string $Title
 * @property string $NumberOfSpaces
 * @property string $NumberOfGuests
 * @property string $Size
 * @property bool $CateringAvailable
 * @property int $Sort
 * @method Property Property()
 * @method HasManyList Galleries()
 */
class EventRoom extends DataObject implements PermissionProvider
{

	const PERMISSIONS_KEY = 'MANAGE_EVENT_ROOMS';

	private static string $table_name = 'EventRoom';

	private static array $extensions = [
		ContentExtension::class,
		Versioned::class,
		FluentVersionedExtension::class,
	];

	private static array $db = [
		'Title' => 'Varchar',
		'NumberOfSpaces' => 'Varchar',
		'NumberOfGuests' => 'Varchar',
		'Size' => 'Varchar',
		'CateringAvailable' => 'Boolean',
		'Sort' => 'Int',
	];

	private static array $has_one = [
		'Property' => Property::class,
	];

	private static array $has_many = [
		'Galleries' => Gallery::class,
	];

	private static array $translate = [
		'Title',
	];

	private static string $default_sort = 'Sort';

	private static array $hidden_fields = [
		'PropertyID',
		'Sort',
	];

	public function getCMSFields(): FieldList
	{
		$this->beforeUpdateCMSFields(function (FieldList $fields) {
			SecondaryTabSet::addFields($fields, [
				TextField::create('Title'),
				TextField::create('NumberOfSpaces'),
				TextField::create('NumberOfGuests'),
				TextField::create('Size'),
				CheckboxField::create('CateringAvailable'),
			]);
		});

		$fields = parent::getCMSFields();
		ModelUtilities::hideFields($this, $fields);
		return $fields;
	}

	public function onAfterWrite(): void
	{
		Gallery::createDefaultGalleries($this);
		parent::onAfterWrite();
	}

	public function CMSEditLink(): string
	{
		return Controller::join_links([
			$this->Property()->CMSEditLink(),
			'ItemEditForm',
			'field',
			'EventRooms',
			'item',
			$this->ID,
		]);
	}

	public function providePermissions(): array
	{
		$permissions = [
			self::PERMISSIONS_KEY => [
				'name' => 'Manage event rooms',
				'category' => 'Properties',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canEdit($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canDelete($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canCreate($member = null, $context = array()): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function useInheritedWorkflow(): bool
	{
		return true;
	}

	public function workflowParent(): DataObject
	{
		return $this->Property();
	}

	public function WorkflowLink(): string
	{
		return Controller::join_links([Director::absoluteBaseURL(), $this->CMSEditLink()]);
	}

}
