<?php

namespace Fastrack\Core\Model;

use Fastrack\Core\Util\ModelUtilities;
use Fastrack\Core\Util\Slug;
use Psr\Container\NotFoundExceptionInterface;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Control\HTTPRequest;
use SilverStripe\Core\Config\Config;
use SilverStripe\Core\Injector\Injector;
use SilverStripe\Forms\CheckboxSetField;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\GridField\GridField;
use SilverStripe\Forms\GridField\GridFieldAddExistingAutocompleter;
use SilverStripe\Forms\GridField\GridFieldConfig;
use SilverStripe\Forms\ListboxField;
use SilverStripe\Forms\LookupField;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\HasManyList;
use SilverStripe\ORM\ManyManyList;
use SilverStripe\ORM\SS_List;
use SilverStripe\ORM\ValidationException;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Versioned\Versioned;
use Symbiote\GridFieldExtensions\GridFieldOrderableRows;
use TractorCow\Fluent\Extension\FluentVersionedExtension;

/**
 * @property string Title
 * @property string Key
 * @method Property Property()
 * @method Room Room()
 * @method HasManyList Features()
 */
class FeatureGroup extends DataObject implements PermissionProvider
{
	const PERMISSIONS_KEY = 'MANAGE_FEATURE_GROUPS';

	private static string $table_name = 'FeatureGroup';

	private static array $extensions = [
		Versioned::class,
		FluentVersionedExtension::class,
	];

	private static array $db = [
		'Title' => 'Varchar',
		'Key' => 'Varchar',
	];

	private static array $has_one = [
		'Property' => Property::class,
		'Room' => Room::class,
	];

	private static array $has_many = [
		'Features' => Feature::class,
	];

	private static array $indexes = [
		'Key' => true,
	];

	private static array $translate = [
		'Title',
	];

	private static array $hidden_fields = [
		'PropertyID',
		'RoomID',
	];

	private static array $summary_fields = [
		'Title',
		'Key',
	];

	private static array $owns = [
		'Features',
	];

	public function getCMSFields(): FieldList
	{
		$fields = parent::getCMSFields();
		/** @var ?GridField $featuresGrid */
		$featuresGrid = $fields->dataFieldByName('Features');
		if ($featuresGrid) {
			/** @var GridFieldConfig $config */
			$config = $featuresGrid->getConfig();
			$config->addComponent(GridFieldOrderableRows::create());
			$config->removeComponentsByType(GridFieldAddExistingAutocompleter::class);
			$fields->replaceField('Features', $featuresGrid);
		}

		$sharedFeatures = SharedFeature::get()->map();
		$selected = $this->Features()->filter('SharedFeatureID:not', 'null')->map('SharedFeatureID')->keys();
		$checkbox = ListboxField::create('SharedFeatures', 'Shared Features', $sharedFeatures, $selected)
			->setDisabledItems($selected)
			->setDescription('You can create shared features from [Properties] > [Features]');
		$fields->addFieldToTab('Root.Features', $checkbox);

		ModelUtilities::hideFields(self::class, $fields);
		return $fields;
	}

	public function onBeforeWrite(): void
	{
		if (!Injector::inst()->has(HTTPRequest::class)) {
			parent::onBeforeWrite();
			return;
		}
		try {
			/** @var HTTPRequest $rq */
			$rq = Injector::inst()->get(HTTPRequest::class);
			$chosen = is_array($rq->postVar('SharedFeatures')) ? $rq->postVar('SharedFeatures') : [];
			$existing = $this->Features()->filter('SharedFeatureID:not', 'null')->map('SharedFeatureID')->keys();
			$additionalIds = array_diff($chosen, $existing);
			foreach ($additionalIds as $additionalId) {
				try {
					$feature = Feature::create();
					$feature->SharedFeatureID = $additionalId;
					$feature->FeatureGroupID = $this->ID;
					$feature->write();
					$feature->publishSingle();
				} catch (ValidationException) {}
			}
		} catch (NotFoundExceptionInterface) {}
		parent::onBeforeWrite();
	}

	public static function createDefaultFeatureGroups(DataObject $dataObject): void
	{
		$featureGroupKeys = Config::inst()->get($dataObject->getClassName(), 'default_feature_groups');
		if (!$featureGroupKeys) return;
		foreach ($featureGroupKeys as $featureGroupKey) {
			self::createFeatureGroup($dataObject, $featureGroupKey);
		}
	}

	private static function createFeatureGroup(DataObject $dataObject, string $featureGroupKey): void
	{
		if (!array_key_exists('FeatureGroups', $dataObject->hasMany())) return;
		if ($dataObject->FeatureGroups()->find('Key', $featureGroupKey)) return;
		try {
			$featureGroup = new FeatureGroup();
			$featureGroup->setField('Title', Slug::deSlug($featureGroupKey));
			$featureGroup->setField('Key', trim($featureGroupKey));
			$featureGroup->write();
			$dataObject->FeatureGroups()->add($featureGroup);
			$featureGroup->publishSingle();
		} catch (ValidationException) {}
	}

	public function providePermissions(): array
	{
		$permissions = [
			self::PERMISSIONS_KEY => [
				'name' => 'Manage feature groups',
				'category' => 'Features',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canEdit($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canDelete($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canCreate($member = null, $context = array()): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

}
