<?php

namespace Fastrack\Core\Model;

use Fastrack\Core\Util\ModelUtilities;
use Fastrack\Core\Util\Slug;
use SilverStripe\AssetAdmin\Forms\UploadField;
use SilverStripe\Core\Config\Config;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\GridField\GridField;
use SilverStripe\Forms\GridField\GridField_ActionMenu;
use SilverStripe\Forms\GridField\GridFieldButtonRow;
use SilverStripe\Forms\GridField\GridFieldConfig;
use SilverStripe\Forms\GridField\GridFieldConfig_RecordEditor;
use SilverStripe\Forms\GridField\GridFieldDataColumns;
use SilverStripe\Forms\GridField\GridFieldDeleteAction;
use SilverStripe\Forms\GridField\GridFieldDetailForm;
use SilverStripe\Forms\GridField\GridFieldEditButton;
use SilverStripe\Forms\GridField\GridFieldFilterHeader;
use SilverStripe\Forms\GridField\GridFieldToolbarHeader;
use SilverStripe\Forms\TextField;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\HasManyList;
use SilverStripe\ORM\ValidationException;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Versioned\Versioned;
use Symbiote\GridFieldExtensions\GridFieldAddNewInlineButton;
use Symbiote\GridFieldExtensions\GridFieldEditableColumns;
use Symbiote\GridFieldExtensions\GridFieldOrderableRows;
use Symbiote\GridFieldExtensions\GridFieldTitleHeader;
use TractorCow\Fluent\Extension\FluentVersionedExtension;

/**
 * @property string $Name
 * @property string $Key
 * @property string $Description
 * @method CalendarEvent CalendarEvent()
 * @method Property Property()
 * @method Restaurant Restaurant()
 * @method Room Room()
 * @method EventRoom EventRoom()
 * @method Offer Offer()
 * @method HasManyList Slides()
 */
class Gallery extends DataObject implements PermissionProvider
{
	const GALLERY_HELP_MESSAGE = '<strong>Note: </strong> After adding as slide you must click save/publish before uploading an image';

	const PERMISSIONS_KEY = 'MANAGE_GALLERIES';

	private static string $table_name = 'Gallery';

	private static array $extensions = [
		Versioned::class,
		FluentVersionedExtension::class,
	];

	private static array $db = [
		'Title' => 'Varchar',
		'Description' => 'Text',
		'Key' => 'Varchar'
	];

	private static array $has_one = [
		'CalendarEvent' => CalendarEvent::class,
		'Property' => Property::class,
		'Restaurant' => Restaurant::class,
		'Room' => Room::class,
		'EventRoom' => EventRoom::class,
		'Offer' => Offer::class,
	];

	private static array $has_many = [
		'Slides' => Slide::class,
	];

	private static array $indexes = [
		'Key' => true,
	];

	private static array $translate = [
		'Title',
		'Name',
		'Description',
	];

	private static array $hidden_fields = [
		'CalendarEventID',
		'PropertyID',
		'RestaurantID',
		'RoomID',
		'EventRoomID',
		'OfferID',
	];

	private static array $owns = [
		'Slides',
	];

	private static array $summary_fields = [
		'Title',
		'Key',
	];

	public function getCMSFields(): FieldList
	{
		$fields = parent::getCMSFields();
		$fields->removeByName(['Slides']);
		$slidesField = GridField::create('Slides', 'Slides', $this->Slides(), self::getSlideGridConfig());
		$fields->addFieldToTab('Root.Main', $slidesField);
		ModelUtilities::hideFields(self::class, $fields);
		return $fields;
	}

	public static function getSlideGridConfig(): GridFieldConfig
	{
		$gridConfiguration = GridFieldConfig_RecordEditor::create(100);
		$gridConfiguration->addComponent(GridFieldOrderableRows::create());
		$gridConfiguration->removeComponentsByType(GridFieldFilterHeader::class);
		/** @var ?GridFieldDataColumns $columns */
		$columns = $gridConfiguration->getComponentByType(GridFieldDataColumns::class);
		$columns?->setDisplayFields([
			'Image.CMSThumbnail' => 'Image',
			'Title' => _t('File.Title', 'Title'),
		]);
		return $gridConfiguration;
	}

	public static function createDefaultGalleries(DataObject $dataObject): void
	{
		$galleryKeys = Config::inst()->get($dataObject->getClassName(), 'default_galleries');
		if (!$galleryKeys) return;
		foreach ($galleryKeys as $galleryKey) {
			self::createGallery($dataObject, $galleryKey);
		}
	}

	private static function createGallery(DataObject $dataObject, string $galleryKey): void
	{
		if (!array_key_exists('Galleries', $dataObject->hasMany())) return;
		if ($dataObject->Galleries()->find('Key', $galleryKey)) return;
		try {
			$gallery = new Gallery();
			$gallery->setField('Title', Slug::deSlug($galleryKey));
			$gallery->setField('Key', trim($galleryKey));
			$gallery->write();
			$dataObject->Galleries()->add($gallery);
			$gallery->publishSingle();
		} catch (ValidationException) {}
	}

	public function providePermissions(): array
	{
		$permissions = [
			self::PERMISSIONS_KEY => [
				'name' => 'Manage galleries',
				'category' => 'Gallery',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canEdit($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canDelete($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canCreate($member = null, $context = array()): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

}
