<?php

namespace Fastrack\Core\Model;

use League\Flysystem\AwsS3V3\AwsS3V3Adapter;
use League\Flysystem\Config;
use Psr\Container\NotFoundExceptionInterface;
use SilverStripe\Assets\Storage\AssetContainer;
use SilverStripe\Assets\Storage\AssetStore;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Assets\Image;
use Imgix\UrlBuilder;
use SilverStripe\Core\Environment;
use SilverStripe\Core\Injector\Injector;
use SilverStripe\View\SSViewer;

/**
 * @property int $ImageWidth
 * @property int $ImageHeight
 */
class Imgix extends Image
{

	const DEFAULT_WIDTH = 992;

	private static string $table_name = 'Imgix';

	private static string $sub_domain = '';

	private static string $secure_url_token = '';

	private static string $folder_path = 'assets/';

	private static array $default_parameters = [
		'auto' => 'compress,format',
	];

	private static array $src_set_widths = [300, 576, 768, 992, 1920];

	private static bool $use_imgix = true;

	private static array $casting = [
		'Tag' => 'HTMLText',
	];

	private static array $db = [
		'ImageWidth' => 'Int',
		'ImageHeight' => 'Int',
	];

	private array $parameters = [];

	private bool $isResponsive = true;

	private string $classes = '';
	private string $sizes = 'auto';


	protected function onBeforeWrite(): void
	{
		parent::onBeforeWrite();
		if ($this->isInDB() && $this->exists()) {
			if (!$this->ImageWidth) {
				$this->ImageWidth = parent::getWidth();
			}
			if (!$this->ImageHeight) {
				$this->ImageHeight = parent::getHeight();
			}
		}
	}

	public function getTag(): string
	{
		if (!$this->isInDB()) {
			return '';
		}
		if (!$this->useImgix()) {
			return parent::getTag();
		}
		return (string) $this->renderWith(SSViewer::get_templates_by_class($this, '', __CLASS__), [
			'IsResponsive' => $this->isResponsive,
		]);
	}

	protected function getDefaultAttributes(): array
	{
		$additionalAttributes = [];
		if (!$this->useImgix()) {
			$additionalAttributes['src'] = $this->getURL();
		} else if ($this->isResponsive) {
			$this->addExtraClass('lazyload');
			$widths = $this->getSetting('src_set_widths');
			$options = ['widths' => $widths];
			$srcSet = $this->builder()->createSrcSet($this->getFolderPath(), $this->getParameters(), $options);
			$additionalAttributes['data-sizes'] = $this->sizes;
			$additionalAttributes['data-srcset'] = $srcSet;
			$this->setDimensions(end($widths));
		} else {
			$additionalAttributes['src'] = $this->getURL(false);
		}
		$attributes = [
			'width' => $this->getWidth(),
			'height' => $this->getHeight(),
			'alt' => $this->getTitle(),
			'class' => $this->getExtraClasses(),
		];
		return array_merge($additionalAttributes, $attributes);
	}

	public function getURL($grant = true): string
	{
		if (!$this->useImgix()) {
			return parent::getURL($grant) ?? '';
		}
		if (!$this->isPublished()) {
			return $this->getPreSignedUrl();
		}
		return $this->builder()->createURL($this->getFolderPath(), $this->getParameters()) ?? '';
	}

	private function builder(): UrlBuilder
	{
		$subDomain = $this->getSetting('sub_domain');
		if (!$subDomain) {
			user_error("Undefined sub_domain: Please add Imgix sub_domain in your config", E_USER_ERROR);
		}
		$builder = new UrlBuilder("$subDomain.imgix.net");
		$builder->setUseHttps(Director::is_https());
		$builder->setSignKey($this->getSetting('secure_url_token'));
		return $builder;
	}

	public function Link(): string
	{
		return $this->getURL();
	}

	public function forTemplate(): string
	{
		return $this->getTag();
	}

	public function addExtraClass(string $classes): static
	{
		$this->classes = $this->classes . ' ' . $classes;
		return $this;
	}

	public function Css(string $classes): static
	{
		return $this->addExtraClass($classes);
	}

	public function getExtraClasses(): string
	{
		$classes = [];
		foreach (explode(' ', $this->classes) as $class) {
			$classes[] = trim($class);
		}
		return join(' ', array_unique($classes));
	}

	/**
	 * Manually set the <img> sizes property. Default is auto
	 *
	 * Option 1:
	 * Use a set preset such as full, half, or quarter to automatically
	 * create the size specification for you. Anything under the specified
	 * $breakpoint will be full sized
	 *
	 * Option 2:
	 * Set specific sizes using presets: xs, sm, md, lg or xl.
	 * E.g. '(max-width: sm) sm' will be '(max-width: 576px) 576px'
	 *
	 * Option 3:
	 * Set sizes without any changes:
	 * E.g. '(max-width: 576px) 576px'
	 *
	 * @param string $sizes
	 * @param string $breakpoint
	 * @return $this
	 */
	public function setSizes(string $sizes, string $breakpoint = 'lg'): static
	{
		$breakpoints = [
			'xs' => 300,
			'sm' => 576,
			'md' => 768,
			'lg' => 992,
			'xl' => 1920,
		];
		$presets = [
			'full' => 1,
			'half' => 0.5,
			'quarter' => 0.25,
		];
		if (array_key_exists($sizes, $presets)) {
			$conditions = [];
			foreach	($breakpoints as $maxWidth) {
				$width = ($breakpoints[$breakpoint] <= $maxWidth) ? $maxWidth * $presets[$sizes] : $maxWidth;
				$maxWidth = $maxWidth === $breakpoints['xl'] ? $maxWidth * 2 : $maxWidth;
				$conditions[] = "(max-width: {$maxWidth}px) {$width}px";
			}
			$this->sizes = join(', ', $conditions);
			return $this;
		}
		array_walk($breakpoints, fn (&$v) => $v = "{$v}px");
		$this->sizes = str_replace(array_keys($breakpoints), array_values($breakpoints), $sizes);
		return $this;
	}

	public function SetResponsive(bool $isResponsive): static
	{
		$this->isResponsive = $isResponsive;
		return $this;
	}

	public function setParameter($key, $value, $append = false): static
	{
		$parameterValue = ($append)
			? $this->getParameter($key) . ',' . $value
			: $value;
		$this->parameters[$key] = $parameterValue;
		return $this;
	}

	public function getParameters(): array
	{
		$defaults = $this->config()->get('default_parameters');
		return array_merge($defaults, $this->parameters);
	}

	public function getParameter($key): string
	{
		$parameters = $this->getParameters();
		if (key_exists($key, $parameters)) {
			return $parameters[$key];
		}
		return '';
	}

	public function setDimensions($width = null, $height = null): static
	{
		if (!is_null($width)) {
			$this->setParameter('w', $width);
		}
		if (!is_null($height)) {
			$this->setParameter('h', $height);
		}
		return $this;
	}

	public function Format(): static
	{
		$this->setParameter('auto', 'format');
		return $this;
	}

	public function Compress(): static
	{
		$this->setParameter('auto', 'compress', true);
		return $this;
	}

	public function Enhance(): static
	{
		$this->setParameter('auto', 'enhance', true);
		return $this;
	}

	public function existingOnly(): static|AssetContainer
	{
		if (!$this->useImgix()) {
			return parent::CMSThumbnail();
		}
		return $this;
	}

	public function CMSThumbnail(): static|AssetContainer
	{
		$this->SetResponsive(false);
		if (!$this->useImgix()) {
			return parent::CMSThumbnail();
		}
		$width = (int)$this->config()->get('cms_thumbnail_width');
		$height = (int)$this->config()->get('cms_thumbnail_height');
		return $this->Fill($width, $height);
	}

	public function StripThumbnail(): static|AssetContainer
	{
		$this->SetResponsive(false);
		if (!$this->useImgix()) {
			return parent::StripThumbnail();
		}
		$width = (int)$this->config()->get('strip_thumbnail_width');
		$height = (int)$this->config()->get('strip_thumbnail_height');
		return $this->Fill($width, $height);
	}

	public function Fit($width, $height): static|AssetContainer
	{
		if (!$this->useImgix()) {
			return parent::Fit($width, $height);
		}
		$this->setDimensions($width, $height);
		$this->setParameter('fit','clip');
		return $this;
	}

	public function FitMax($width, $height): static|AssetContainer|null
	{
		if (!$this->useImgix()) {
			return parent::FitMax($width, $height);
		}
		$this->setParameter('fit','crop');
		$this->setDimensions($width, $height);
		return $this;
	}


	public function Fill($width, $height): static|AssetContainer
	{
		if (!$this->useImgix()) {
			return parent::Fill($width, $height) ?? $this;
		}
		$this->setDimensions($width, $height);
		$this->setParameter('fit','crop');
		return $this;
	}

	public function FillMax($width, $height): static|AssetContainer
	{
		if (!$this->useImgix()) {
			return parent::FillMax($width, $height);
		}
		$this->Fill($width, $height);
		$this->setParameter('max-w', $this->getWidth());
		$this->setParameter('max-h', $this->getHeight());
		return $this;
	}

	public function Pad($width, $height, $backgroundColor='FFFFFF', $transparencyPercent = 0): static|AssetContainer
	{
		if (!$this->useImgix()) {
			return parent::Pad($width, $height, $backgroundColor, $transparencyPercent);
		}
		$this->setDimensions($width, $height);
		$this->setParameter('fit','fill');
		$this->setParameter('bg', $backgroundColor);
		return $this;
	}

	public function ScaleWidth($width): static|AssetContainer
	{
		if (!$this->useImgix()) {
			return parent::ScaleWidth($width);
		}
		$this->setDimensions($width);
		$this->setParameter('fit','clip');
		return $this;
	}

	public function ScaleMaxWidth($width): static|AssetContainer
	{
		if (!$this->useImgix()) {
			return parent::ScaleMaxWidth($width);
		}
		$this->ScaleWidth($width);
		$this->setParameter('max-w', $this->getWidth());
		return $this;
	}

	public function ScaleHeight($height): static|AssetContainer
	{
		if (!$this->useImgix()) {
			return parent::ScaleHeight($height);
		}
		$this->setDimensions(null, $height);
		$this->setParameter('fit','clip');
		return $this;
	}

	public function ScaleMaxHeight($height): static|AssetContainer
	{
		if (!$this->useImgix()) {
			return parent::ScaleMaxHeight($height);
		}
		$this->ScaleHeight($height);
		$this->setParameter('max-h', $this->getHeight());
		return $this;
	}

	public function CropWidth($width): static|AssetContainer
	{
		if (!$this->useImgix()) {
			return parent::CropWidth($width);
		}
		if ($this->getWidth() > $width) {
			$this->Fill($width, $this->getHeight());
		}
		return $this;
	}

	public function CropHeight($height): static|AssetContainer
	{
		if (!$this->useImgix()) {
			return parent::CropHeight($height);
		}
		if ($this->getHeight() > $height) {
			$this->Fill($height, $this->getWidth());
		}
		return $this;
	}

	public function isSize($width, $height): bool
	{
		return $this->isWidth($width) && $this->isHeight($height);
	}

	public function isWidth($width): bool
	{
		return !empty($width) && $this->getWidth() == $width;
	}

	public function isHeight($height): bool
	{
		return !empty($height) && $this->getHeight() == $height;
	}

	public function getWidth(): int
	{
		$width = $this->ImageWidth ?: 0;
		$parameter = (int) $this->getParameter('w');
		return $parameter ?: $width;
	}

	public function getHeight(): int
	{
		$setHeight = (int) $this->getParameter('h');
		if ($setHeight) {
			return $setHeight;
		}
		$setWidth = (int) $this->getParameter('w');
		$originalWidth = $this->ImageWidth ?: 0;
		$originalHeight = $this->ImageHeight ?: 0;
		if ($setWidth && $originalWidth && $originalHeight) {
			$ratio = $originalHeight / $originalWidth;
			return intval(round($setWidth * $ratio));
		}
		return $originalHeight;
	}

	private function getSetting(string $key): mixed
	{
		$value = $this->config()->get($key);
		if (is_string($value) && preg_match('/^`(?<name>[^`]+)`$/', $value, $matches)) {
			$envValue = Environment::getEnv($matches['name']);
			if ($envValue !== false) {
				$value = $envValue;
			} elseif (defined($matches['name'])) {
				$value = constant($matches['name']);
			} else {
				$value = null;
			}
		}
		return $value;
	}

	public function useImgix() : bool
	{
		if (!$this->getSetting('use_imgix')) {
			return false;
		}
		return !in_array(strtolower($this->getExtension()), ['gif', 'svg', 'ico']);
	}

	private function getFolderPath(): string
	{
		$prefix = $this->isPublished()
			? Environment::getEnv('AWS_PUBLIC_BUCKET_PREFIX')
			: Environment::getEnv('AWS_PROTECTED_BUCKET_PREFIX');
		$relativePath = $prefix
			? Controller::join_links([$prefix, $this->Filename])
			: Director::makeRelative(parent::getURL());
		return str_ireplace($this->getSetting('folder_path'), '', $relativePath);
	}

	private function getPreSignedUrl(): string
	{
		$expire = new \DateTime('+1 hour');
		try {
			/** @var AwsS3V3Adapter $protectedAdapter */
			$protectedAdapter = Injector::inst()->get(AwsS3V3Adapter::class . '.protected');
			/** @var AssetStore $store */
			$store = Injector::inst()->get(AssetStore::class);
			$absPath = $store->getAsURL($this->generateFilename(), $this->getHash(), $this->getVariant());
			$protectedBucket = Environment::getEnv('AWS_PROTECTED_BUCKET_PREFIX');
			$pos = stripos($absPath, $protectedBucket);
			if (is_bool($pos)) {
				return '';
			}
			$filePath = substr($absPath, $pos + strlen($protectedBucket));
			return $protectedAdapter->temporaryUrl($filePath, $expire, new Config());
		} catch (NotFoundExceptionInterface $e) {
			return '';
		}
	}

}
