<?php


namespace Fastrack\Core\Model;

use Fastrack\Core\Util\FormUtil;
use League\OAuth2\Client\Provider\Instagram;
use League\OAuth2\Client\Token\AccessToken;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Forms\LiteralField;
use SilverStripe\Forms\TextField;
use SilverStripe\Forms\ToggleCompositeField;
use SilverStripe\ORM\DataObject;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\SiteConfig\SiteConfig;

/**
 * @property string $AppId
 * @property string $AppSecret
 * @property string $UserName
 * @property string $State
 * @property string $AccessToken
 * @property string $RefreshToken
 * @property string $ResourceOwnerId
 * @property int $Expires
 * @method SiteConfig SiteConfig()
 */
class InstagramToken extends DataObject implements PermissionProvider
{

	const PERMISSIONS_KEY = 'MANAGE_INSTAGRAM_TOKENS_ALL';

	private static string $table_name = 'InstagramToken';

	private static array $db = [
		'AppId' => 'Varchar',
		'AppSecret' => 'Varchar',
		'UserName' => 'Varchar',
		'State' => 'Varchar',
		'AccessToken' => 'Varchar',
		'RefreshToken' => 'Varchar',
		'ResourceOwnerId' => 'Varchar',
		'Expires' => 'Int',
	];

	private static array $has_one = [
		'SiteConfig' => SiteConfig::class,
	];

	private static array $summary_fields = [
		'UserName',
	];

	public function getTitle(): string
	{
		return $this->UserName;
	}

	public function getProvider(): Instagram
	{
		$redirectUrl = Controller::join_links([
			Director::absoluteBaseURL(),
			'instagram',
			$this->UserName,
			'auth',
		]);
		return new Instagram([
			'clientId' => $this->AppId,
			'clientSecret' => $this->AppSecret,
			'redirectUri' => $redirectUrl,
		]);
	}

	public function getToken(): AccessToken
	{
		return new AccessToken([
			'access_token' => $this->AccessToken,
			'refresh_token' => $this->RefreshToken,
			'expires_in' => $this->Expires,
			'resource_owner_id' => $this->ResourceOwnerId,
		]);
	}


	public function getCMSFields()
	{
		$fields = parent::getCMSFields();
		$fields->removeByName([
			'State',
			'AccessToken',
			'RefreshToken',
			'ResourceOwnerId',
			'Expires',
			'SiteConfigID',
		]);
		$tokenEndpoint = $this->getTokenEndpoint();
		$authEndpoint = $this->getAuthEndpoint();
		$tokenLink = "<a href=\"$tokenEndpoint\" target='_blank'>$tokenEndpoint</a>";
		$authLink = "<a href=\"$authEndpoint\" target='_blank'>$authEndpoint</a>";

		$fields->addFieldToTab('Root.Main', ToggleCompositeField::create('AuthenticationHeader', 'Authentication', [
			TextField::create('AccessToken'),
			TextField::create('RefreshToken'),
			TextField::create('ResourceOwnerId'),
			TextField::create('Expires'),
			LiteralField::create('URL', FormUtil::infoField('Token endpoint', $tokenLink)),
			LiteralField::create('URL', FormUtil::infoField('Auth endpoint', $authLink)),
		]));
		return $fields;
	}


	public function providePermissions(): array
	{
		$permissions = [
			self::PERMISSIONS_KEY => [
				'name' => 'Manage Instagram tokens',
				'category' => 'Settings'
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canEdit($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canDelete($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canCreate($member = null, $context = array()): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	private function getTokenEndpoint(): string
	{
		return Controller::join_links([
			Director::absoluteBaseURL(),
			'instagram',
			$this->UserName,
			'access-token',
		]);
	}

	private function getAuthEndpoint(): string
	{
		return Controller::join_links([
			Director::absoluteBaseURL(),
			'instagram',
			$this->UserName,
			'auth',
		]);
	}

}
