<?php

namespace Fastrack\Core\Model;

use Fastrack\Core\Admin\PropertyAdmin;
use Fastrack\Core\Extension\ContentExtension;
use Fastrack\Core\Extension\SecondaryTabSet;
use Fastrack\Core\Util\ModelUtilities;
use Psr\Container\NotFoundExceptionInterface;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Core\Injector\Injector;
use SilverStripe\Forms\CheckboxField;
use SilverStripe\Forms\DateField;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\GridField\GridField;
use SilverStripe\Forms\GridField\GridFieldAddNewButton;
use SilverStripe\Forms\GridField\GridFieldConfig_RelationEditor;
use SilverStripe\Forms\GridField\GridFieldEditButton;
use SilverStripe\Forms\HTMLEditor\HTMLEditorField;
use SilverStripe\Forms\TextField;
use SilverStripe\ORM\ArrayList;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\FieldType\DBDate;
use SilverStripe\ORM\HasManyList;
use SilverStripe\ORM\ManyManyList;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Versioned\Versioned;
use Symbiote\GridFieldExtensions\GridFieldAddExistingSearchButton;
use Symbiote\GridFieldExtensions\GridFieldEditableColumns;
use TractorCow\Fluent\Extension\FluentVersionedExtension;

/**
 *
 * @see ContentExtension
 * @property string $Slug
 * @property string $Excerpt
 * @property string $Content
 * @method Imgix Image()
 *
 * @property string Title
 * @property string Price
 * @property string Url
 * @property DBDate Start
 * @property DBDate End
 * @property string Terms
 * @property bool Featured
 * @method ManyManyList Properties()
 * @method HasManyList Galleries()
 */
class Offer extends DataObject implements PermissionProvider
{

	const PERMISSIONS_KEY = 'MANAGE_OFFERS';

	private static string $table_name = 'Offer';

	private static array $extensions = [
		ContentExtension::class,
		Versioned::class,
		FluentVersionedExtension::class,
	];

	private static array $db = [
		'Title' => 'Varchar',
		'Price' => 'Varchar',
		'Url' => 'Varchar',
		'Start' => 'Date',
		'End' => 'Date',
		'Terms' => 'HTMLText',
		'Featured' => 'Boolean',
	];

	private static array $has_many = [
		'Galleries' => Gallery::class,
	];

	private static array $many_many = [
		'Properties' => Property::class,
	];

	private static array $many_many_extraFields = [
		'Properties' => [
			'OfferSortOrder' => 'Int',
			'Url' => 'Varchar',
		],
	];

	private static array $translate = [
		'Title',
		'Terms',
	];

	private static array $hidden_fields = [];

	private static array $summary_fields = [
		'Title' => 'Title',
		'OfferProperties' => 'Property',
	];

	public function OfferProperties(): string
	{
		$properties = join(', ', $this->Properties()->map()->values());
		if (strlen($properties) > 50) {
			return substr($properties, 0, 50) . '...';
		}
		return $properties;
	}

	public function getCMSFields(): FieldList
	{
		$this->beforeUpdateCMSFields(function (FieldList $fields) {
			SecondaryTabSet::addFields($fields, [
				TextField::create('Title'),
				TextField::create('Price'),
				TextField::create('Url')->setTitle('URL'),
				CheckboxField::create('Featured'),
				DateField::create('Start'),
				DateField::create('End'),
				HTMLEditorField::create('Terms'),
			]);
		});

		$fields = parent::getCMSFields();
		$gridConfiguration = GridFieldConfig_RelationEditor::create();
		$gridConfiguration->removeComponentsByType(GridFieldAddNewButton::class);
		$gridConfiguration->removeComponentsByType(GridFieldEditButton::class);
		$gridConfiguration->addComponent(GridFieldAddExistingSearchButton::create());
		$gridConfiguration->addComponent(GridFieldEditableColumns::create()->setDisplayFields(['Url' => [
			'title' => 'URL',
			'field' => TextField::class,
		]]));
		$grid = new GridField('Properties', 'Properties', $this->Properties(), $gridConfiguration);
		$fields->addFieldToTab('Root.Properties', $grid);

		ModelUtilities::hideFields($this, $fields);
		return $fields;
	}

	public function onAfterWrite(): void
	{
		Gallery::createDefaultGalleries($this);
		parent::onAfterWrite();
	}

	public function isValid(): bool
	{
		$today = strtotime(date('Y-m-d'));
		if ($this->End && ($today > strtotime($this->End))) {
			return false;
		}
		if ($this->Start && ($today < strtotime($this->Start))) {
			return false;
		}
		return $this->Properties()->count() > 0;
	}

	public static function getValidOffers(): ArrayList
	{
		return Offer::get()->filterByCallback(function (Offer $offer) {
			return $offer->isValid();
		});
	}

	public function providePermissions(): array
	{
		$permissions = [
			self::PERMISSIONS_KEY => [
				'name' => 'Manage offers',
				'category' => 'Properties',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canEdit($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canDelete($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canCreate($member = null, $context = array()): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

}
