<?php

namespace Fastrack\Core\Model;

use Exception;
use Fastrack\Core\Util\ModelUtilities;
use Fastrack\Core\Util\PaymentGateway\CreateSessionRequest;
use Fastrack\Core\Util\PaymentGateway\Windcave;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\GridField\GridField;
use SilverStripe\Forms\GridField\GridFieldAddNewButton;
use SilverStripe\Forms\GridField\GridFieldConfig_RelationEditor;
use SilverStripe\Forms\GridField\GridFieldEditButton;
use SilverStripe\Forms\LiteralField;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\HasManyList;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Versioned\Versioned;
use Symbiote\GridFieldExtensions\GridFieldAddExistingSearchButton;

/**
 * @property string $Name
 * @property string $Type
 * @property bool $IsUAT
 * @property string $Username
 * @property string $AccessKey
 * @method HasManyList Properties()
 */
class PaymentGateway extends DataObject implements PermissionProvider
{

	const PERMISSIONS_KEY = 'MANAGE_PAYMENT_GATEWAYS';

	private static string $table_name = 'PaymentGateway';

	private static array $extensions = [
		Versioned::class,
	];

	private static array $db = [
		'Name' => 'Varchar',
		'Type' => "Enum('Windcave')",
		'IsUAT' => 'Boolean',
		'Username' => 'Varchar',
		'AccessKey' => 'Varchar',
	];

	private static array $has_many = [
		'Properties' => Property::class,
	];

	private static array $hidden_fields = [
		'Name',
	];

	public function getCMSFields(): FieldList
	{
		$fields = parent::getCMSFields();
		$gridConfig = GridFieldConfig_RelationEditor::create(100);
		$gridConfig->removeComponentsByType(GridFieldAddNewButton::class);
		$gridConfig->removeComponentsByType(GridFieldEditButton::class);
		$gridConfig->addComponent(new GridFieldAddExistingSearchButton());
		$properties = new GridField('Properties', 'Properties', $this->Properties(), $gridConfig);
		$fields->addFieldToTab('Root.Properties', $properties);

		$isValid = $this->isValid() ?
			'<strong style="color:#56bd59">Settings are valid</strong>' :
			'<strong style="color:#da273b">Settings are invalid</strong>';

		$fields->addFieldsToTab('Root.Main', [
			LiteralField::create('IsValid', $isValid),
		]);
		ModelUtilities::hideFields($this, $fields);
		return $fields;
	}

	protected function onBeforeWrite(): void
	{
		$this->Name = "$this->Type - $this->Username";
		parent::onBeforeWrite();
	}


	public function getUrl(): string
	{
		if ($this->Type === 'Windcave') {
			$mode = $this->IsUAT ? 'uat' : 'sec';
			return "https://$mode.windcave.com/api/v1/sessions";
		}
		return '';
	}

	private function isValid(): bool
	{
		if ($this->Type !== 'Windcave') {
			return false;
		}
		if (!$this->Username || !$this->AccessKey) {
			return false;
		}
		$request = new CreateSessionRequest();
		$request->SetType('auth');
		$request->SetAmount('1.00');
		$request->SetCurrency('AUD');
		$request->SetMerchantReference(uniqid());
		$request->SetApproved('https://test.com');
		$request->SetDeclined('https://test.com');
		$request->SetCancelled('https://test.com');
		$request->SetFirstName('Test');
		$request->SetLastName('Test');
		$request->SetEmail('test@test.com');
		$request->SetPhoneNumber('00000000000');
		$request->SetSurcharge('0.00');

		$windcave = new Windcave();
		try {
			$windcave->createSession($this, $request);
			return true;
		} catch (Exception) {
			return false;
		}
	}

	public function providePermissions(): array
	{
		$permissions = [
			self::PERMISSIONS_KEY => [
				'name' => 'Manage payment gateways',
				'category' => 'Payment',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canEdit($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canDelete($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canCreate($member = null, $context = array()): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

}
