<?php

namespace Fastrack\Core\Model;

use Ballen\Distical\Calculator;
use Ballen\Distical\Entities\LatLong;
use Exception;
use Fastrack\Core\Extension\ContentExtension;
use Fastrack\Core\Extension\LocationExtension;
use Fastrack\Core\Extension\SecondaryTabSet;
use Fastrack\Core\Util\ModelUtilities;
use Psr\Container\NotFoundExceptionInterface;
use Psr\Log\LoggerInterface;
use SilverStripe\Core\Config\Config;
use SilverStripe\Core\Injector\Injector;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\GridField\GridField;
use SilverStripe\Forms\GridField\GridFieldAddNewButton;
use SilverStripe\Forms\GridField\GridFieldConfig_RelationEditor;
use SilverStripe\Forms\TextField;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\ManyManyList;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Versioned\Versioned;
use Symbiote\GridFieldExtensions\GridFieldAddExistingSearchButton;
use TractorCow\Fluent\Extension\FluentVersionedExtension;

/**
 * @property string $Title
 * @property string $Website
 * @property string $Phone
 * @property string $Email
 * @method ManyManyList Categories()
 * @method ManyManyList Properties
 *
 * @see LocationExtension
 * @property string Address1
 * @property string Address2
 * @property string City
 * @property string Postcode
 * @property string State
 * @property string Country
 * @property string Directions
 * @property string Latitude
 * @property string Longitude
 * @property string MapUrl
 *
 * @see ContentExtension
 * @property string $Slug
 * @property string $Excerpt
 * @property string $Content
 * @property int ImageID
 * @method Imgix Image()
 */
class Place extends DataObject implements PermissionProvider
{

	const PERMISSIONS_KEY = 'MANAGE_PLACES';

	private static string $table_name = 'Place';

	private static array $extensions = [
		ContentExtension::class,
		LocationExtension::class,
		Versioned::class,
		FluentVersionedExtension::class,
	];

	private static array $db = [
		'Title' => 'Varchar',
		'Website' => 'Varchar',
		'Phone' => 'Varchar',
		'Email' => 'Varchar',
	];

	private static array $many_many = [
		'Categories' => PlaceCategory::class,
		'Properties' => Property::class,
	];

	private static array $translate = [
		'Title',
	];

	private static bool $auto_relate_places = true;

	private static array $hidden_fields = [
		'Properties',
	];

	public function getCMSFields(): FieldList
	{
		$this->beforeUpdateCMSFields(function (FieldList $fields) {
			SecondaryTabSet::addFields($fields, [
				TextField::create('Title'),
				TextField::create('Website'),
				TextField::create('Phone'),
				TextField::create('Email'),
			]);
		});

		$fields = parent::getCMSFields();
		$categoryConfig = GridFieldConfig_RelationEditor::create(20);
		$categoryConfig->removeComponentsByType(GridFieldAddNewButton::class);
		$categoryConfig->addComponent(new GridFieldAddExistingSearchButton());
		$categoriesGrid = GridField::create('Categories', 'Categories', $this->Categories(), $categoryConfig);
		$fields->replaceField('Categories', $categoriesGrid);

		ModelUtilities::hideFields($this, $fields);
		return $fields;
	}

	public function onAfterWrite(): void
	{
		$this->addPropertiesWithinRadius();
		parent::onAfterWrite();
	}

	private function addPropertiesWithinRadius(): void
	{
		if (!Config::inst()->get(Place::class, 'auto_relate_places')) {
			return;
		}
		if ($this->Latitude && $this->Longitude) {
			$placeCoordinates = new LatLong(floatval($this->Latitude), floatval($this->Longitude));
			foreach (Property::get() as $property) {
				$this->Properties()->remove($property);
				if (!$property->Latitude || !$property->Longitude) {
					continue;
				}
				try {
					$propertyCoordinates = new LatLong(floatval($property->Latitude), floatval($property->Longitude));
					$distance = new Calculator($propertyCoordinates, $placeCoordinates);
					$radius = $property->PlaceRadius ?: 50;
					if ($distance->get()->asKilometres() <= $radius) {
						$this->Properties()->add($property);
					}
				} catch (Exception $e) {
					$this->getLogger()?->error($e->getMessage());
				}

			}
		}
	}

	public function providePermissions(): array
	{
		$permissions = [
			self::PERMISSIONS_KEY => [
				'name' => 'Manage places',
				'category' => 'Places',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canEdit($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canDelete($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canCreate($member = null, $context = array()): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	private function getLogger(): ?LoggerInterface
	{
		try {
			return Injector::inst()->get(LoggerInterface::class);
		} catch (NotFoundExceptionInterface) {
			return null;
		}
	}

}
