<?php

namespace Fastrack\Core\Model;

use Ballen\Distical\Calculator;
use Ballen\Distical\Entities\LatLong;
use Exception;
use Fastrack\Core\Admin\PropertyAdmin;
use Fastrack\Core\Extension\ContactExtension;
use Fastrack\Core\Extension\ContentExtension;
use Fastrack\Core\Extension\LocationExtension;
use Fastrack\Core\Extension\SecondaryTabSet;
use Fastrack\Core\Util\ModelUtilities;
use Psr\Container\NotFoundExceptionInterface;
use SilverStripe\Assets\Folder;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Core\Config\Config;
use SilverStripe\Core\Injector\Injector;
use SilverStripe\Forms\CheckboxField;
use SilverStripe\Forms\CurrencyField;
use SilverStripe\Forms\DropdownField;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\FormField;
use SilverStripe\Forms\GridField\GridField;
use SilverStripe\Forms\GridField\GridFieldAddExistingAutocompleter;
use SilverStripe\Forms\GridField\GridFieldAddNewButton;
use SilverStripe\Forms\Tab;
use SilverStripe\Forms\TextField;
use SilverStripe\Forms\TreeDropdownField;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\HasManyList;
use SilverStripe\ORM\ManyManyList;
use SilverStripe\ORM\UnsavedRelationList;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Versioned\Versioned;
use Spatie\SchemaOrg\GeoCoordinates;
use Spatie\SchemaOrg\Hotel;
use Spatie\SchemaOrg\PostalAddress;
use Symbiote\GridFieldExtensions\GridFieldAddExistingSearchButton;
use Symbiote\GridFieldExtensions\GridFieldOrderableRows;
use TractorCow\Fluent\Extension\FluentVersionedExtension;

/**
 * @property string $Title
 * @property string $FromPrice
 * @property bool $Featured
 * @property string $BookingEngineId
 * @property string $BookingLink
 * @property int $PlaceRadius
 * @property int $Sort
 *
 * @see ContentExtension
 * @property string $Slug
 * @property string $Excerpt
 * @property string $Content
 * @method Imgix Image()
 *
 * @see LocationExtension
 * @property string Address1
 * @property string Address2
 * @property string City
 * @property string Postcode
 * @property string State
 * @property string Country
 * @property string Directions
 * @property string Latitude
 * @property string Longitude
 * @property string MapUrl
 *
 * @method string getAddressFormatted($country = false, $lineSeperator = PHP_EOL)
 *
 * @method PaymentGateway PaymentGateway()
 * @method Folder AssetFolder()
 * @method HasManyList Rooms()
 * @method HasManyList Restaurants()
 * @method HasManyList EventRooms()
 * @method HasManyList FeatureGroups()
 * @method HasManyList Galleries()
 * @method HasManyList Reviews()
 * @method ManyManyList Places()
 *
 */
class Property extends DataObject implements PermissionProvider
{

	const PERMISSIONS_KEY = 'MANAGE_PROPERTY_ALL';

	private static string $table_name = 'Property';

	private static array $extensions = [
		ContentExtension::class,
		ContactExtension::class,
		LocationExtension::class,
		Versioned::class,
		FluentVersionedExtension::class,
	];

	private static array $db = [
		'Title' => 'Varchar',
		'FromPrice' => 'Currency',
		'Featured' => 'Boolean',
		'BookingEngineId' => 'Int',
		'BookingLink' => 'Varchar',
		'PlaceRadius' => 'Int',
		'Sort' => 'Int',
	];

	private static array $has_one = [
		'PaymentGateway' => PaymentGateway::class,
		'AssetFolder' => Folder::class,
	];

	private static array $has_many = [
		'Rooms' => Room::class,
		'Restaurants' => Restaurant::class,
		'EventRooms' => EventRoom::class,
		'FeatureGroups' => FeatureGroup::class,
		'Galleries' => Gallery::class,
		'Reviews' => Review::class,
	];

	private static array $belongs_many_many = [
		'Places' => Place::class,
		'Offers' => Offer::class,
	];

	private static array $owns = [];

	private static array $defaults = [
		'PlaceRadius' => 50,
	];

	private static string $default_sort = 'Sort';

	private static array $translate = [
		'Title',
	];

	private static array $hidden_fields = [
		'AssetFolder',
		'Places',
		'Sort',
	];

	public function Offers(): ManyManyList|UnsavedRelationList
	{
		return $this->getManyManyComponents('Offers')->sort('OfferSortOrder');
	}

	public function ConstructBookingLink(): string
	{
		$link = $this->BookingLink;
		$this->extend('updateConstructBookingLink', $link);
		return $link ?? '';
	}

	public function getCMSFields(): FieldList
	{
		$this->beforeUpdateCMSFields(function (FieldList $fields) {
			SecondaryTabSet::addFields($fields, [
				TextField::create('Title'),
				CurrencyField::create('FromPrice'),
				TextField::create('BookingLink'),
				TextField::create('BookingEngineId')
					->setTitle('Booking engine ID'),
				CheckboxField::create('Featured'),
				DropdownField::create('PaymentGatewayID', 'Payment gateway', PaymentGateway::get()->map())
					->setDescription('Create via [Payment Gateway] tab'),
				TextField::create('PlaceRadius')
					->setTitle('Place radius (KM)')
					->setDescription('Distance to associate with places created from the [Places] tab'),
				TreeDropdownField::create('AssetFolderID', 'Files folder', Folder::class)
					->setDescription('Files will be uploaded to this folder'),
			]);
		});

		$fields = parent::getCMSFields();
		/** @var ?GridField $offersGrid */
		$offersGrid = $fields->dataFieldByName('Offers');
		if ($offersGrid) {
				$offersGrid->setDescription('Create offers from [Property] > [Offers] then link them here');
				$offersGrid->getConfig()
						->removeComponentsByType([GridFieldAddNewButton::class, GridFieldAddExistingAutocompleter::class])
						->addComponent(GridFieldAddExistingSearchButton::create())
						->addComponent(GridFieldOrderableRows::create('OfferSortOrder'));
		}
		$ordered = ['Rooms', 'Reviews', 'Restaurants', 'EventRooms'];
		ModelUtilities::addOrderRows($fields, ['Rooms', 'Reviews', 'Restaurants', 'EventRooms']);
		ModelUtilities::removeLinkExisting($fields, array_merge($ordered, ['Galleries', 'FeatureGroups']));
		ModelUtilities::hideFields(self::class, $fields);
		return $fields;
	}

	public function onAfterWrite(): void
	{
		Gallery::createDefaultGalleries($this);
		FeatureGroup::createDefaultFeatureGroups($this);
		parent::onAfterWrite();
	}

	public function providePermissions(): array
	{
		$permissions = [
			self::PERMISSIONS_KEY => [
				'name' => 'Manage all properties',
				'category' => 'Properties',
				'sort' => -100,
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canEdit($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canDelete($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canCreate($member = null, $context = array()): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function getStructuredData(): Hotel
	{
		$postalAddress = new PostalAddress();
		$postalAddress->streetAddress($this->Address1);
		$postalAddress->postalCode($this->Postcode);
		$postalAddress->addressLocality($this->State);
		$postalAddress->addressRegion($this->City);
		$postalAddress->addressCountry($this->Country);

		$geo = new GeoCoordinates();
		$geo->latitude($this->Latitude);
		$geo->longitude($this->Longitude);

		$hotel = new Hotel();
		$hotel->name($this->Title);
		$hotel->description($this->Excerpt);
		$hotel->address($postalAddress);
		$hotel->telephone($this->Phone);
		$hotel->email($this->Email);
		$hotel->image($this->Image->Link());
		$hotel->geo($geo);
		if ($this->FromPrice) {
			$hotel->priceRange($this->FromPrice);
		}
		if ($this->MapUrl) {
			$hotel->hasMap($this->MapUrl);
		}
		return $hotel;
	}

	protected function onBeforeWrite(): void
	{
		$this->addPlacesWithinRadius();
		parent::onBeforeWrite();
	}


	private function addPlacesWithinRadius(): void
	{
		if (!Config::inst()->get(Place::class, 'auto_relate_places')) {
			return;
		}
		if ($this->Latitude && $this->Longitude) {
			$propertyCoordinates = new LatLong($this->Latitude, $this->Longitude);
			/** @var Property $property */
			$property = Property::get()->byID($this->ID);
			if (!$property) {
				return;
			}
			$places = Place::get();
			foreach ($places as $place) {
				$property->Places()->remove($place);
				if ($place->Latitude && $place->Longitude) {
					try {
						$placeCoordinates = new LatLong($place->Latitude, $place->Longitude);
						$distance = new Calculator($placeCoordinates, $propertyCoordinates);
						$radius = $this->PlaceRadius ? $this->PlaceRadius : 50;
						if ($distance->get()->asKilometres() <= $radius) {
							$property->Places()->add($place);
						}
					} catch (Exception) {}
				}
			}
		}
	}

}
