<?php

namespace Fastrack\Core\Model;

use Fastrack\Core\Extension\ContactExtension;
use Fastrack\Core\Extension\ContentExtension;
use Fastrack\Core\Extension\LocationExtension;
use Fastrack\Core\Extension\SecondaryTabSet;
use Fastrack\Core\Util\ModelUtilities;
use SilverStripe\Control\Controller;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\TextField;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\HasManyList;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Versioned\Versioned;
use TractorCow\Fluent\Extension\FluentVersionedExtension;

/**
 * @property string $Title
 * @property int $Sort
 * @method Property Property()
 * @method HasManyList Galleries()
 *
 * @see ContentExtension
 * @property string $Slug
 * @property string $Excerpt
 * @property string $Content
 * @method Imgix Image()
 *
 * @see ContactExtension
 * @property string $Email
 * @property string $Phone
 * @property string $Facebook
 * @property string $Twitter
 * @property string $Instagram
 * @property string $Youtube
 * @property string $TripAdvisor
 * @property string $LinkedIn
 */
class Restaurant extends DataObject implements PermissionProvider
{

	const PERMISSIONS_KEY = 'MANAGE_RESTAURANTS';

	private static string $table_name = 'Restaurant';

	private static array $extensions = [
		ContactExtension::class,
		ContentExtension::class,
		LocationExtension::class,
		Versioned::class,
		FluentVersionedExtension::class,
	];

	private static array $db = [
		'Title' => 'Varchar',
		'Sort' => 'Int',
	];

	private static array $has_one = [
		'Property' => Property::class,
	];

	private static array $has_many = [
		'Galleries' => Gallery::class,
	];

	private static array $translate = [
		'Title',
	];

	private static array $indexes = [
		'Sort' => true,
	];

	private static string $default_sort = 'Sort';

	private static array $hidden_fields = [
		'Sort',
		'PropertyID'
	];

	protected function onAfterWrite(): void
	{
		Gallery::createDefaultGalleries($this);
		parent::onAfterWrite();
	}

	public function getCMSFields(): FieldList
	{
		$this->beforeUpdateCMSFields(function (FieldList $fields) {
			SecondaryTabSet::addFields($fields, [
				TextField::create('Title'),
			]);
		});
		$fields = parent::getCMSFields();
		ModelUtilities::hideFields($this, $fields);
		return $fields;
	}

	public function CMSEditLink(): string
	{
		return Controller::join_links([
			$this->Property()->CMSEditLink(),
			'ItemEditForm',
			'field',
			'Restaurants',
			'item',
			$this->ID,
		]);
	}

	public function providePermissions(): array
	{
		$permissions = [
			self::PERMISSIONS_KEY => [
				'name' => 'Manage restaurants',
				'category' => 'Properties',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canEdit($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canDelete($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canCreate($member = null, $context = array()): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

}
