<?php

namespace Fastrack\Core\Model;

use Fastrack\Core\Extension\ContentExtension;
use Fastrack\Core\Extension\SecondaryTabSet;
use Fastrack\Core\Util\ModelUtilities;
use SilverStripe\Forms\CheckboxField;
use SilverStripe\Forms\CurrencyField;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\NumericField;
use SilverStripe\Forms\TextField;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\HasManyList;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Versioned\Versioned;
use TractorCow\Fluent\Extension\FluentVersionedExtension;

/**
 * @see ContentExtension
 * @property string $Slug
 * @property string $Excerpt
 * @property string $Content
 * @method Imgix Image()
 *
 * @property string $Title
 * @property int $NumberOfRooms
 * @property int $NumberOfBathrooms
 * @property int $NumberOfBeds
 * @property int $NumberOfGuests
 * @property int $MinimumNumberOfGuests
 * @property bool $PetsAllowed
 * @property bool $SmokingAllowed
 * @property string $FromPrice
 * @property string $BookingEngineId
 * @property string $BookingLink
 * @property bool $Featured
 * @property int $Sort
 * @method Property Property()
 * @method HasManyList FeatureGroups()
 * @method HasManyList Galleries()
 */
class Room extends DataObject implements PermissionProvider
{

	const PERMISSIONS_KEY = 'MANAGE_ROOMS';

	private static string $table_name = 'Room';

	private static array $extensions = [
		ContentExtension::class,
		Versioned::class,
		FluentVersionedExtension::class,
	];

	private static array $db = [
		'Title' => 'Varchar',
		'NumberOfRooms' => 'Int',
		'NumberOfBathrooms' => 'Int',
		'NumberOfBeds' => 'Int',
		'NumberOfGuests' => 'Int',
		'MinimumNumberOfGuests' => 'Int',
		'PetsAllowed' => 'Boolean',
		'SmokingAllowed' => 'Boolean',
		'FromPrice' => 'Currency',
		'BookingEngineId' => 'Varchar',
		'BookingLink' => 'Varchar',
		'Featured' => 'Boolean',
		'Sort' => 'Int',
	];

	private static array $has_one = [
		'Property' => Property::class,
	];

	private static array $has_many = [
		'FeatureGroups' => FeatureGroup::class,
		'Galleries' => Gallery::class,
	];

	private static string $default_sort = 'Sort';

	private static array $indexes = [
		'Sort' => true,
	];

	private static array $translate = [
		'Title',
		'Name',
	];

	private static array $hidden_fields = [
		'PropertyID',
		'Sort',
	];

	public function getCMSFields(): FieldList
	{
		$this->beforeUpdateCMSFields(function (FieldList $fields) {
			$generalFields = [
				TextField::create('Title'),
				NumericField::create('NumberOfRooms'),
				NumericField::create('NumberOfBathrooms'),
				NumericField::create('NumberOfBeds'),
				NumericField::create('NumberOfGuests'),
				NumericField::create('MinimumNumberOfGuests'),
				CurrencyField::create('FromPrice'),
				TextField::create('BookingEngineId')
					->setTitle('Booking Engine ID'),
				TextField::create('BookingLink'),
				CheckboxField::create('Featured'),
				CheckboxField::create('PetsAllowed'),
				CheckboxField::create('SmokingAllowed'),
			];
			SecondaryTabSet::addFields($fields, $generalFields);
		});
		$fields = parent::getCMSFields();
		ModelUtilities::hideFields(self::class, $fields);
		return $fields;
	}

	public function onAfterWrite(): void
	{
		Gallery::createDefaultGalleries($this);
		FeatureGroup::createDefaultFeatureGroups($this);
		parent::onAfterWrite();
	}

	public function ConstructBookingLink(): string
	{
		$link = $this->BookingLink;
		if ($link) {
			$this->extend('updateConstructBookingLink', $link);
			return $link;
		}
		return $this->Property()->ConstructBookingLink();
	}

	public function providePermissions(): array
	{
		$permissions = [
			self::PERMISSIONS_KEY => [
				'name' => 'Manage rooms',
				'category' => 'Properties',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canEdit($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canDelete($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canCreate($member = null, $context = array()): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

}
