<?php

namespace Fastrack\Core\Util;

use League\Flysystem\FileAttributes;
use Psr\Container\NotFoundExceptionInterface;
use Psr\SimpleCache\CacheInterface;
use Psr\SimpleCache\InvalidArgumentException;
use SilverStripe\Core\Flushable;
use SilverStripe\Core\Injector\Injector;

class AssetCache implements AssetCacheInterface, Flushable
{
	const UNKNOWN = -1;
	const NOT_EXISTS = 0;
	const EXISTS = 1;
	const CACHE_NAME = '.Fastrack.Core.Util.AssetCache';

	private CacheInterface $cache;
	private bool $isPublic;

	/**
	 * @throws NotFoundExceptionInterface
	 */
	public function __construct(bool $isPublic)
	{
		$this->isPublic = $isPublic;
		$this->cache = Injector::inst()->get(CacheInterface::class . self::CACHE_NAME);
	}

	/**
	 * @throws NotFoundExceptionInterface
	 * @throws InvalidArgumentException
	 */
	public function getFile(string $path): CachedFile
	{
		$cacheKey = $this->cachedFileKey($path);
		$cached = $this->cache->get($cacheKey);
		if (is_null($cached)) {
			$file = new CachedFile(self::UNKNOWN, new FileAttributes($path));
			$this->setFile($path, $file);
			return $file;
		}
		return CachedFile::deserialize($cached);
	}

	/**
	 * @throws InvalidArgumentException
	 */
	public function setFile(string $path, CachedFile $file): void
	{
		$this->cache->set($this->cachedFileKey($path), $file->serialize());
	}

	/**
	 * @throws InvalidArgumentException
	 */
	public function deleteFile(string $path): void
	{
		foreach ($this->cacheKeys($path, true) as $key) {
			$this->cache->delete($key);
		}
	}

	/**
	 * @throws InvalidArgumentException
	 */
	public function copyFile(string $source, string $destination): void
	{
		$this->deleteFile($source);
		$this->deleteFile($destination);
	}

	/**
	 * @throws InvalidArgumentException
	 */
	public function moveFile(string $source, string $destination): void
	{
		$this->deleteFile($source);
		$this->deleteFile($destination);
	}

	/**
	 * @throws NotFoundExceptionInterface
	 * @throws InvalidArgumentException
	 */
	public function getDirectory(string $path): CachedDirectory
	{
		$cacheKey = $this->cachedDirectoryKey($path);
		$cached = $this->cache->get($cacheKey);
		if (is_null($cached)) {
			$directory = new CachedDirectory(self::UNKNOWN);
			$this->setDirectory($path, $directory);
			return $directory;
		}
		return CachedDirectory::deserialize($cached);
	}

	/**
	 * @throws InvalidArgumentException
	 */
	public function setDirectory(string $path, CachedDirectory $directory): void
	{
		$this->cache->set($this->cachedDirectoryKey($path), $directory->serialize());
	}

	/**
	 * @throws InvalidArgumentException
	 */
	public function deleteDirectory(string $path): void
	{
		foreach ($this->cacheKeys($path, false) as $key) {
			$this->cache->delete($key);
		}
	}

	private function cachedFileKey($path): string
	{
		$visibility = $this->isPublic ? 'public' : 'protected';
		return $this->cacheKeys($path, true)[$visibility];
	}

	private function cachedDirectoryKey($path): string
	{
		$visibility = $this->isPublic ? 'public' : 'protected';
		return $this->cacheKeys($path, false)[$visibility];
	}

	private function cacheKeys($path, $isFile): array
	{
		$type = $isFile ? 'file' : 'directory';
		return [
			'protected' => "protected-$type-$path",
			'public' => "public-$type-$path",
		];
	}

	public static function flush(): void
	{
		/** @var CacheInterface $cache */
		$cache = Injector::inst()->get(CacheInterface::class . self::CACHE_NAME);
		$cache->clear();
	}
}
