<?php

namespace Fastrack\Core\Util;

use League\Flysystem\Config;
use League\Flysystem\FileAttributes;
use League\Flysystem\FilesystemAdapter;
use League\Flysystem\StorageAttributes;

abstract class CachedAdapter implements FilesystemAdapter
{

	protected FilesystemAdapter $adapter;
	protected AssetCacheInterface $cache;

	public function __construct(FilesystemAdapter $adapter, AssetCacheInterface $cache)
	{
		$this->adapter = $adapter;
		$this->cache = $cache;
	}

	protected function getUrl($path)
	{
		$cached = $this->cache->getFile($path);
		if ($cached->url()) {
			return $cached->url();
		}
		if ($path && method_exists($this->adapter, 'publicUrl')) {
			$url = $this->adapter->publicUrl($path, new Config());
			$file = new CachedFile($cached->exists(), $cached->getFileAttributes(), $cached->getContents(), $url);
			$this->cache->setFile($path, $file);
			return $url;
		}
		return $path;
	}

	public function fileExists(string $path): bool
	{
		$cached = $this->cache->getFile($path);
		if ($cached->exists() !== AssetCache::UNKNOWN) {
			return $cached->exists() === AssetCache::EXISTS;
		}
		$adapterResponse = $this->adapter->fileExists($path);
		$exists = $adapterResponse ? AssetCache::EXISTS : AssetCache::NOT_EXISTS;
		$file = new CachedFile($exists, $cached->getFileAttributes(), $cached->getContents(), $cached->url());
		$this->cache->setFile($path, $file);
		return $adapterResponse;
	}

	public function write(string $path, string $contents, Config $config): void
	{
		$this->adapter->write($path, $contents, $config);
		$cached = $this->cache->getFile($path);
		$file = new CachedFile(AssetCache::EXISTS, $cached->getFileAttributes(), $contents, $cached->url());
		$this->cache->setFile($path, $file);
	}

	public function writeStream(string $path, $contents, Config $config): void
	{
		$this->adapter->writeStream($path, $contents, $config);
		$cached = $this->cache->getFile($path);
		$file = new CachedFile(AssetCache::EXISTS, $cached->getFileAttributes(), '', $cached->url());
		$this->cache->setFile($path, $file);
	}

	public function read(string $path): string
	{
		$cached = $this->cache->getFile($path);
		if ($cached->getContents()) {
			return $cached->getContents();
		}
		$contents = $this->adapter->read($path);
		$file = new CachedFile($cached->exists(), $cached->getFileAttributes(), $contents, $cached->url());
		$this->cache->setFile($path, $file);
		return $contents;
	}

	public function readStream(string $path)
	{
		return $this->adapter->readStream($path);
	}

	public function delete(string $path): void
	{
		$this->adapter->delete($path);
		$this->cache->deleteFile($path);
	}

	public function deleteDirectory(string $path): void
	{
		$this->adapter->deleteDirectory($path);
		$this->cache->deleteDirectory($path);
	}

	public function createDirectory(string $path, Config $config): void
	{
		$this->adapter->createDirectory($path, $config);
		$cached = new CachedDirectory(AssetCache::EXISTS);
		$this->cache->setDirectory($path, $cached);
	}

	public function directoryExists(string $path): bool
	{
		$cached = $this->cache->getDirectory($path);
		if ($cached->exists() !== AssetCache::UNKNOWN) {
			return $cached->exists() === AssetCache::EXISTS;
		}
		$adapterResponse = $this->adapter->directoryExists($path);
		$exists = $adapterResponse ? AssetCache::EXISTS : AssetCache::NOT_EXISTS;
		$directory = new CachedDirectory($exists);
		$this->cache->setDirectory($path, $directory);
		return $adapterResponse;
	}

	public function setVisibility(string $path, string $visibility): void
	{
		$this->adapter->setVisibility($path, $visibility);
		$visibilityAttributes = $this->adapter->visibility($path);
		$this->updateAttributes($path, $visibilityAttributes, StorageAttributes::ATTRIBUTE_VISIBILITY);
	}

	public function visibility(string $path): FileAttributes
	{
		$cached = $this->cache->getFile($path);
		if ($cached->getFileAttributes()->visibility()) {
			return $cached->getFileAttributes();
		}
		$visibility = $this->adapter->visibility($path);
		return $this->updateAttributes($path, $visibility, StorageAttributes::ATTRIBUTE_VISIBILITY);

	}

	public function mimeType(string $path): FileAttributes
	{
		$cached = $this->cache->getFile($path);
		if ($cached->getFileAttributes()->mimeType()) {
			return $cached->getFileAttributes();
		}
		$mimeType = $this->adapter->mimeType($path);
		return $this->updateAttributes($path, $mimeType, StorageAttributes::ATTRIBUTE_MIME_TYPE);
	}

	public function lastModified(string $path): FileAttributes
	{
		$cached = $this->cache->getFile($path);
		if ($cached->getFileAttributes()->lastModified()) {
			return $cached->getFileAttributes();
		}
		$lastModified = $this->adapter->lastModified($path);
		return $this->updateAttributes($path, $lastModified, StorageAttributes::ATTRIBUTE_LAST_MODIFIED);
	}

	public function fileSize(string $path): FileAttributes
	{
		$cached = $this->cache->getFile($path);
		if ($cached->getFileAttributes()->fileSize()) {
			return $cached->getFileAttributes();
		}
		$fileSize = $this->adapter->fileSize($path);
		return $this->updateAttributes($path, $fileSize, StorageAttributes::ATTRIBUTE_FILE_SIZE);
	}

	public function listContents(string $path, bool $deep): iterable
	{
		return $this->adapter->listContents($path, $deep);
	}

	public function move(string $source, string $destination, Config $config): void
	{
		$this->adapter->move($source, $destination, $config);
		$this->cache->moveFile($source, $destination);
	}

	public function copy(string $source, string $destination, Config $config): void
	{
		$this->adapter->copy($source, $destination, $config);
		$this->cache->copyFile($source, $destination);
	}

	private function updateAttributes(string $path, FileAttributes $attributes, string $attribute): FileAttributes
	{
		$cached = $this->cache->getFile($path);
		$data = $cached->getFileAttributes()->jsonSerialize();
		$data[$attribute] = match ($attribute) {
			StorageAttributes::ATTRIBUTE_TYPE => $attributes->type(),
			StorageAttributes::ATTRIBUTE_PATH => $attributes->path(),
			StorageAttributes::ATTRIBUTE_FILE_SIZE => $attributes->fileSize(),
			StorageAttributes::ATTRIBUTE_VISIBILITY => $attributes->visibility(),
			StorageAttributes::ATTRIBUTE_LAST_MODIFIED => $attributes->lastModified(),
			StorageAttributes::ATTRIBUTE_MIME_TYPE => $attributes->mimeType(),
			StorageAttributes::ATTRIBUTE_EXTRA_METADATA => $attributes->extraMetadata(),
		};
		$data[StorageAttributes::ATTRIBUTE_EXTRA_METADATA] =
			array_merge($data[StorageAttributes::ATTRIBUTE_EXTRA_METADATA], $attributes->extraMetadata());
		$attributes = FileAttributes::fromArray($data);
		$file = new CachedFile($cached->exists(), $attributes, $cached->getContents(), $cached->url());
		$this->cache->setFile($path, $file);
		return $attributes;
	}

}
