<?php

namespace Fastrack\Core\Util\Seekom;

use Exception;
use Fastrack\Core\Util\Vault;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use League\OAuth2\Client\Provider\Exception\IdentityProviderException;
use League\OAuth2\Client\Provider\GenericProvider;
use Psr\Container\NotFoundExceptionInterface;
use Psr\Log\LoggerInterface;
use SilverStripe\Control\Controller;
use SilverStripe\Control\HTTPResponse;
use SilverStripe\Control\HTTPResponse_Exception;
use SilverStripe\Core\Environment;
use SilverStripe\Core\Injector\Injector;

class SeekomController extends Controller
{

	const CSRF_TOKEN = 'state';
	const AUTH_CODE = 'code';
	const SESSION_KEY = 'seekom_oath_2_state';

	private static array $allowed_actions = ['auth'];
	private static array $url_handlers = ['auth' => 'auth'];

	private GenericProvider $provider;
	private Vault $vault;
	private SeekomCredentials $credentials;

	/**
	 * @throws Exception
	 */
	public function __construct()
	{
		parent::__construct();
		$this->vault = new Vault(new Client());
		$this->credentials = $this->getCredentials();
		$this->setProvider();
	}

	/**
	 * @throws HTTPResponse_Exception
	 * @throws Exception
	 */
	public function auth(): HTTPResponse
	{
		if ($this->credentials->hasAccessToken()) {
			$this->refreshToken();
			$this->setMessage('REFRESHED');
		} else if (!$this->getRequest()->getVar(self::AUTH_CODE)) {
			return $this->getResponse()->redirect($this->getAuthUrl());
		} else if (!$this->isValidState()) {
			$this->httpError(400, 'INVALID_STATE');
		} else {
			$this->acquireToken();
			$this->setMessage('ACQUIRED');
		}
		return $this->getResponse()->setStatusCode(200);
	}

	private function getAuthUrl(): string
	{
		$authUrl = $this->provider->getAuthorizationUrl();
		$this->getRequest()->getSession()->set(self::SESSION_KEY, $this->provider->getState());
		return $authUrl;
	}

	/**
	 * @throws Exception
	 */
	private function refreshToken(): void
	{
		try {
			$accessToken = $this->credentials->getAccessToken();
			$newAccessToken = $this->provider->getAccessToken('refresh_token', [
				'refresh_token' => $accessToken->getRefreshToken(),
			]);
			$this->credentials->setAccessToken($newAccessToken);
			$this->saveCredentials();
		} catch (IdentityProviderException $e) {
			$this->getLogger()?->error($e->getMessage());
			throw new Exception('Could not refresh token');
		}
	}

	/**
	 * @throws Exception
	 */
	private function acquireToken(): void
	{
		try {
			$accessToken = $this->provider->getAccessToken('authorization_code', [
				'code' => $this->getRequest()->getVar(self::AUTH_CODE),
				'grant_type' => 'authorization_code',
			]);
			$this->credentials->setAccessToken($accessToken);
			$this->saveCredentials();
		} catch (IdentityProviderException $e) {
			$this->getLogger()?->error($e->getMessage());
			throw new Exception('Could not acquire token');
		}
	}

	private function setProvider(): void
	{
		$collaborators = ['optionProvider' => new SeekomOptionProvider()];
		$this->provider = new GenericProvider([
			'clientId' => $this->credentials->clientId,
			'clientSecret' => $this->credentials->clientSecret,
			'redirectUri' => Environment::getEnv('SS_BASE_URL') . '/ibex/auth',
			'urlAuthorize' => Controller::join_links([$this->credentials->authUrl, '/oauth/auth']),
			'urlAccessToken' => Controller::join_links([$this->credentials->apiUrl, '/oauth/token']),
			'urlResourceOwnerDetails' => Controller::join_links([$this->credentials->apiUrl, '/oauth/resource']),
		], $collaborators);
	}

	private function isValidState(): bool
	{
		$requestState = $this->getRequest()->getVar(self::CSRF_TOKEN);
		$sessionState = $this->getRequest()->getSession()->get(self::SESSION_KEY);
		if (!$requestState || !$sessionState) {
			return false;
		}
		return $requestState === $sessionState;
	}

	/**
	 * @throws Exception
	 */
	private function getCredentials(): SeekomCredentials
	{
		try {
			$secret = $this->vault->getSecret('clients', SeekomCredentials::getVaultPath());
		return new SeekomCredentials($secret);
		} catch (GuzzleException $e) {
			$this->getLogger()?->error($e->getMessage());
			throw new Exception('Could not fetch credentials');
		}
	}

	/**
	 * @throws Exception
	 */
	private function saveCredentials(): void
	{
		try {
			$this->vault->setSecret('clients', SeekomCredentials::getVaultPath(), $this->credentials->getData());
		} catch (GuzzleException $e) {
			$this->getLogger()?->error($e->getMessage());
			throw new Exception('Could not save credential');
		}
	}

	private function setMessage(string $message): void
	{
		$this->getResponse()->setBody($message . PHP_EOL)->addHeader('Content-Type', 'text/plain');
	}

	private function getLogger(): ?LoggerInterface
	{
		try {
			return Injector::inst()->get(LoggerInterface::class);
		} catch (NotFoundExceptionInterface) {
			return null;
		}
	}

}
