<?php

namespace Fastrack\Core\Util;

use Exception;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\GuzzleException;
use Psr\Container\NotFoundExceptionInterface;
use Psr\Http\Message\ResponseInterface;
use Psr\Log\LoggerInterface;
use Psr\SimpleCache\CacheInterface;
use Psr\SimpleCache\InvalidArgumentException;
use SilverStripe\Core\Environment;
use SilverStripe\Core\Injector\Injector;
use Throwable;

class Vault
{

	private ClientInterface $client;
	private CacheInterface $cache;
	private LoggerInterface $logger;
	private string $vaultUrl;
	private string $roleId;
	private string $secretId;
	private string $vaultToken;
	const TOKEN_CACHE_KEY = 'token';


	/**
	 * @throws Exception
	 */
	public function __construct(ClientInterface $client)
	{
		$this->client = $client;
		$this->vaultUrl = Environment::getEnv('VAULT_URL');
		$this->roleId = Environment::getEnv('VAULT_ROLE_ID');
		$this->secretId = Environment::getEnv('VAULT_SECRET_ID');
		try {
			$this->cache = Cache::getCache();
			$this->logger = Injector::inst()->get(LoggerInterface::class);
			$this->vaultToken = $this->getToken();
		} catch (Throwable|InvalidArgumentException $e) {
			$this->getLogger()?->error($e->getMessage());
			throw new Exception('Could construct Vault');
		}
	}

	/**
	 * @param string $secret
	 * @param string $path
	 * @return mixed
	 * @throws GuzzleException
	 * @throws Exception
	 */
	public function getSecret(string $secret, string $path): array
	{
		$url = $this->vaultUrl . "/v1/$secret/data/$path";
		$response = $this->client->request('GET', $url, [
			'headers' => [
				'X-Vault-Request' => 'true',
				'X-Vault-Token' => $this->vaultToken,
			]
		]);
		if ($response->getStatusCode() === 200) {
			$body = json_decode($response->getBody(), true);
			return $body['data']['data'];
		}
		throw new Exception($this->handleErrors($response));
	}

	/**
	 * @param string $secret
	 * @param string $path
	 * @param $value
	 * @throws GuzzleException|Exception
	 */
	public function setSecret(string $secret, string $path, $value): void
	{
		$url = $this->vaultUrl . "/v1/$secret/data/$path";
		$response = $this->client->request('POST', $url, [
			'headers' => [
				'X-Vault-Request' => 'true',
				'X-Vault-Token' => $this->vaultToken,
			],
			'json' => ['data' => $value],
		]);
		if ($response->getReasonPhrase() !== 'OK') {
			throw new Exception($this->handleErrors($response));
		}
	}

	private function handleErrors(ResponseInterface $response): string
	{
		$message = 'An unexpected error occurred';
		$body = $response->getBody();
		if ($body['errors']) {
			$message = join(', ', $body['errors']);
		}
		$this->logger->error($message);
		return $message;
	}

	/**
	 * @throws GuzzleException
	 * @throws InvalidArgumentException
	 * @throws Exception|NotFoundExceptionInterface
	 */
	private function getToken()
	{
		$token = $this->cache->get(self::TOKEN_CACHE_KEY);
		if ($token) {
			return $token;
		}
		$url = $this->vaultUrl . "/v1/auth/approle/login";
		$response = $this->client->request('POST', $url, [
			'json' => [
				'role_id' => $this->roleId,
				'secret_id' => $this->secretId,
			],
		]);
		if ($response->getReasonPhrase() === 'OK') {
			$body = json_decode($response->getBody(), true);
			$token = $body['auth']['client_token'];
			$lease = $body['auth']['lease_duration'];
			$this->cache->set(self::TOKEN_CACHE_KEY, $token, $lease);
			return $token;
		}
		throw new Exception($this->handleErrors($response));
	}

	private function getLogger(): ?LoggerInterface
	{
		try {
			return Injector::inst()->get(LoggerInterface::class);
		} catch (NotFoundExceptionInterface) {
			return null;
		}
	}

}
